# -*- coding: utf-8 -*-

from __future__ import print_function

import MDAnalysis
import numpy
import numpy.linalg
import math
from pyanca import KabschAlign 
import numpy as np
from mpl_toolkits.mplot3d import Axes3D
import matplotlib.pyplot as plt
import logging

log = logging.getLogger('main.IterativeMeansAlign');
log.setLevel(logging.DEBUG);

class IterativeMeansAlign(object):
	
	def __init__(self):
		"""
		Constructor
		"""

	def iterativeMeans(self, coords, eps, maxIter, mapped=False, fname='na',shape=[0,0,0]):

		if mapped:
			coords = np.memmap(fname, dtype='float64', mode='r+').reshape(shape) 
		# all coordinates are expected to be passed as a (Ns x 3 x Na)  array
		# where Na = number of atoms; Ns = number of snapshots
	
		# This file has been edited to produce identical results as the original matlab implementation.

		Ns = numpy.shape(coords)[0];
		dim = numpy.shape(coords)[1];
		Na = numpy.shape(coords)[2];
		
		log.debug('Shape of array in IterativeMeans: {0}'.format(numpy.shape(coords)));
		
		avgCoords = [];			# track average coordinates
		kalign = KabschAlign.KabschAlign();		# initialize for use

		ok = 0;				# tracking convergence of iterative means
		itr = 1; 			# iteration number
		
		eRMSD = [];
		"""
		fig = plt.figure();
		ax = fig.gca(projection='3d');
		plt.ion();"""
		while not(ok):
			tmpRMSD = [];
			mnC = numpy.mean(coords, 0); 
			avgCoords.append(mnC);
			for i in range(0,Ns):
				fromXYZ = coords[i];
				[R, T, xRMSD, err] = kalign.kabsch(mnC, fromXYZ);
				tmpRMSD.append(xRMSD); 
				tmp = numpy.tile(T.flatten(), (Na,1)).T;
				pxyz = numpy.dot(R,fromXYZ) + tmp;  
				coords[i,:,:] = pxyz;
			eRMSD.append(numpy.array(tmpRMSD).T);
			newMnC = numpy.mean(coords,0); 
			err = math.sqrt(sum( (mnC.flatten()-newMnC.flatten())**2) )
			log.info('Iteration #{0} with an error of {1}'.format(itr, err))
			if err <= eps or itr == maxIter:
				ok = 1;
			itr = itr + 1;
		if mapped:
			del coords;
			coords = 0;
		return [itr,avgCoords,eRMSD,coords];

if __name__=='__main__':
	u = MDAnalysis.Universe('../data/ubq_1111.pdb', '../data/UBQ_500ns.dcd', permissive=False);
	ca = u.selectAtoms('name CA');
	cacoords = []; frames = [];
	for ts in u.trajectory:
		f = ca.coordinates();
		cacoords.append(f.T);
		frames.append(ts.frame);
	print((numpy.shape(cacoords)));
	
	iterAlign = IterativeMeansAlign();
	[itr, avgCoords, eRMSD, newCoords] = iterAlign.iterativeMeans(cacoords, 0.001, 4, True); 
	
	print((numpy.shape(eRMSD)));
	newCoords = numpy.array(newCoords);
	newCoords = numpy.reshape(newCoords, (10000,3*69)); print((numpy.shape(newCoords))); 
	fig = plt.figure();
	ax = fig.add_subplot(111);
	ax.plot(frames, eRMSD[1], linestyle='solid', linewidth=2.0);
	plt.show();




# -*- coding: utf-8 -*-

"""
SD2

This module contains only one function, SD2, which performs spatial decorrelation of 2nd order.
"""


from __future__ import print_function

import numpy
from sys import stdout 
from numpy import argsort, cov, diag, dot, sqrt, absolute, arange
from numpy.linalg import inv, eig, eigh
from numpy import sqrt, ndarray, matrix, float64

#perform 2nd order spatial decorrelation
def SD2(data,m= None,verbose=True):
    
    """
    SD2 - Spatial Decorrelation of 2nd order of real signals 
    
    Parameters:
    
        data -- a 3n x T data matrix (number 3 is due to the x,y,z coordinates for each atom). May be a numpy array or
                matrix where 
				
                n: size of the protein
                T: Number of snapshots of MD trajectory
    
        m -- dimensionality of the subspace we are interested in. Default value is None, in
        which case m=n.
        
        verbose -- print information on progress. Default is True.
    
    Returns:
    
        A 3n x m matrix U (NumPy matrix type), such that Y = U x data is a 2nd order
        spatially whitened source extracted from the 3n x T data matrix 'data'. If m is
        omitted, U is a square 3n x 3n matrix (as many sources as sensors). 
         
        Ds: has eigen values sorted by increasing variance
		PCs: holds the index for m most significant principal components by decreasing variance
        S = Ds[PCs] 
		
		S – Eigen values of the ‘data’ covariance matrix 
        B - Eigen vectors of the 'data' covariance matrix. The eigen vectors are orthogonal.
        """
    
    # GB: we do some checking of the input arguments and copy data to new
    # variables to avoid messing with the original input. We also require double
    # precision (float64) and a numpy matrix type for data.
    
    assert isinstance(data, ndarray),\
        "data (input data matrix) is of the wrong type (%s)" % type(data)
    origtype = data.dtype # remember to return matrix B of the same type
    data = matrix(data.astype(float64))    
    
    assert data.ndim == 2, "X has %d dimensions, should be 2" % data.ndim
    assert (verbose == True) or (verbose == False), \
        "verbose parameter should be either True or False"
    
    [T,n] = data.shape # GB: n is number of input signals, T is number of samples
    
    if m==None:
        m=n 	# Number of sources defaults to # of sensors
    assert m<=n,\
        "SD2 -> Do not ask more sources (%d) than sensors (%d )here!!!" % (m,n)

    if verbose:
        print("2nd order Spatial Decorrelation -> Looking for %d sources" % m, file=stdout)
        print("2nd order Spatial Decorrelation -> Removing the mean value", file=stdout)
    
    data = data.T
    
    # remove the mean from data
    data -= data.mean(1)
    
    # whitening & projection onto signal subspace
    # ===========================================
    if verbose:
        print("2nd order Spatial Decorrelation -> Whitening the data", file=stdout)
    [D,U] = eigh((data * data.T) / float(T)) # An eigen basis for the sample covariance matrix
    k = D.argsort()
    Ds = D[k] # Sort by increasing variances
    PCs = arange(n-1, n-m-1, -1)    # The m most significant princip. comp. by decreasing variance
    S = Ds[PCs]
    # --- PCA  ----------------------------------------------------------
    B = U[:,k[PCs]].T    # % At this stage, B does the PCA on m components
 
    # --- Scaling  ------------------------------------------------------
    scales = sqrt(S) # The scales of the principal components .
    U = diag(1./scales) * B  # Now, B does PCA followed by a rescaling = sphering
    # --- Sphering ------------------------------------------------------

    Y = U * data # %% We have done the easy part: B1 is a whitening matrix and Y is white.
    return (Y, S, B.T, U)

    """
    NOTE: At this stage, Y is spatially whitened by performing PCA analysis on m components of the real data
    Y is now a matrix of spatially uncorrelated components.
    """



# -*- coding: utf-8 -*-

"""
TD4

This module contains only one function, TD4, which does joint diagonalization of cumulant matrices of order 4 which depends on the time delays. It allows us to extract signals which are as independent as possible and which was not obtained while performing SD2, TD2.
"""

from __future__ import print_function

import numpy as np
from sys import stdout 
from numpy import abs, append, arange, arctan2, argsort, array, concatenate, \
                  cos, diag, dot, eye, float32, float64, matrix, multiply, ndarray, newaxis, \
                  sign, sin, sqrt, zeros, ones
from numpy.linalg import eig, pinv
import numpy
import matplotlib.pyplot as plt
import warnings

def TD4(Z, m=None,V=None, lag=None, verbose=True):
 
    """
    TD4 - Temporal Decorrelation of 4th order of real signals 
    
    Parameters:
    
        Z -- an m x T spatially uncorrelated of order 2 and temporally uncorrelated 
        of order 2 matrix (m subspaces, T samples). May be a  numpy array or a matrix where                  
                m: number of subspaces we are interested in
                T: Number of snapshots of MD trajectory
        
        V -- separating matrix obtained after doing the PCA analysis on m components
             of real data followed by temporal decorrelation of the spatially whitened 
             data
        
        lag -- lag time in the form of an integer denoting by time steps
        
        verbose -- print information on progress. Default is True.
    
    Returns:
    
       W -- separating matrix                 
        """
    
    # GB: we do some checking of the input arguments and copy data to new
    # variables to avoid messing with the original input. We also require double
    # precision (float64) and a numpy matrix type for X.
    
    warnings.simplefilter("ignore", np.ComplexWarning)
    [n,T] = Z.shape # GB: n is number of input signals, T is number of samples
    
    if m==None:
        m=n 	# Number of sources defaults to # of sensors
    assert m<=n,\
        "TD4 -> Do not ask more sources (%d) than sensors (%d )here!!!" % (m,n)
        
    print("4th order Temporal Decorrelation -> Estimating cumulant matrices")
    
    # Reshaping of the data, hoping to speed up things a little bit...
    
    Z = Z.T
    #Z = Z - Z.mean()
    dimsymm = int(((m) * (( m )+ 1)) / 2)	# Dim. of the space of real symm matrices
    nbcm = dimsymm  # number of cumulant matrices
    CM = matrix(zeros([(m),(m)*nbcm], dtype=float64)) # Storage for cumulant matrices
    R = matrix(eye((m), dtype=float64))
    Qij = matrix(zeros([m,m], dtype=float64)) # Temp for a cum. matrix
    Xim	= zeros(m , dtype=float64) # Temp
    Xijm = zeros(m , dtype=float64) # Temp
    #Uns = numpy.ones([1,m], dtype=numpy.uint32)    # for convenience
    # GB: we don't translate that one because NumPy doesn't need Tony's rule
    
    # I am using a symmetry trick to save storage.  I should write a short note one of these
    # days explaining what is going on here.
    Range = arange(m) # will index the columns of CM where to store the cum. mats.
    
    # Removing 4th order temporal decorrelations with time delay = lag
    for im in range(m):
        
        Xim = Z[:,im]
        Xijm = multiply(Xim[0 : T - lag :] , Xim[lag : T : ])
             
        # Note to myself: the -R on next line can be removed: it does not affect
        # the joint diagonalization criterion
        
        Qij = dot(multiply(Xijm , Z[0: T-lag :]).T  , Z[lag : T :]) / float(T - lag)  - R - 2 * dot(R[:,im], R[:,im].T)
        
        # To ensure symmetricity of the covariance matrix a mathematical  computation is done
        Qij = (Qij + Qij.T)/2
        CM[:,Range]	= Qij    
        Range = Range  + m 
        for jm in range(im):
                Xijm = multiply(Z[0 : T - lag :, im] , Z[lag : T  :, jm ])
                Qij = sqrt(2) * dot(multiply(Xijm, Z[0 :T - lag : ]).T , Z[lag : T :]) / float(T - lag) \
                    - R[:,im] * R[:,jm].T - R[:,jm] * R[:,im].T
                
                # To ensure symmetricity of the covariance matrix a mathematical  computation is done
                Qij = (Qij + Qij.T)/2
                CM[:,Range]	= Qij
                Range = Range + m 
    
    
    nbcm = int(nbcm)
    # Now we have nbcm = m(m+1)/2 cumulants matrices stored in a big m x m*nbcm array.
    H = matrix(eye(m, dtype=float64))
        
    Diag = zeros(m, dtype=float64)
    On = 0.0
    Range = arange(m)
    for im in range(nbcm):
        Diag = diag(CM[:,Range])
        On = On + (Diag*Diag).sum(axis=0)
        Range = Range + m
    Off = (multiply(CM,CM).sum(axis=0)).sum(axis=0) - On
    
    seuil = 1.0e-6 / (sqrt(T)) # % A statistically scaled threshold on `small" angles
    encore = True
    sweep = 0 # % sweep number
    updates = 0 # % Total number of rotations
    upds = 0 # % Number of rotations in a given s
    g = zeros([2,nbcm], dtype=float64)
    gg = zeros([2,2], dtype=float64)
    G = zeros([2,2], dtype=float64)
    c = 0
    s = 0
    ton	= 0
    toff = 0
    theta = 0
    Gain = 0
    
    # Joint diagonalization proper
    
    if verbose:
        print("TD4 -> Contrast optimization by joint diagonalization", file=stdout)
    
    while encore:
        encore = False
        if verbose:
            print("TD4 -> Sweep #%3d" % sweep, end=' ', file=stdout)
        sweep = sweep + 1
        upds  = 0
        Hkeep = H
      
        for p in range(m-1):
            for q in range(p+1, m):
                
                Ip = arange(p, m*nbcm, m)
                Iq = arange(q, m*nbcm, m)
                
                # computation of Givens angle
                g = concatenate([CM[p,Ip] - CM[q,Iq], CM[p,Iq] + CM[q,Ip]])
                gg = dot(g, g.T)
                ton = gg[0,0] - gg[1,1] 
                toff = gg[0,1] + gg[1,0]
                theta = 0.5 * arctan2(toff, ton + sqrt(ton * ton + toff * toff))
                Gain = (sqrt(ton * ton + toff * toff) - ton) / 4.0
                
                # Givens update
                if abs(theta) > seuil:
                    encore = True
                    upds = upds + 1
                    c = cos(theta) 
                    s = sin(theta)
                    G = matrix([[c, -s] , [s, c] ])
                    pair = array([p,q])
                    H[:,pair] = H[:,pair] * G
                    CM[pair,:] = G.T * CM[pair,:]
                    CM[:,concatenate([Ip,Iq])] = \
                        append( c*CM[:,Ip]+s*CM[:,Iq], -s*CM[:,Ip]+c*CM[:,Iq], \
                               axis=1)
                    On = On + Gain
                    Off = Off - Gain
                    
        if verbose:
            print("completed in %d rotations" % upds, file=stdout)
        updates = updates + upds
    if verbose:
        print("TD4 -> Total of %d Givens rotations" % updates, file=stdout)
    
    # A separating matrix
    # ===================
    
    W = H.T * V
    
    # Permute the rows of the separating matrix B to get the most energetic components first.
    # Here the **signals** are normalized to unit variance.  Therefore, the sort is
    # according to the norm of the columns of A = pinv(W)

    if verbose:
        print("TD4 -> Sorting the components", file=stdout)
    
    A = pinv(W)
    keys =  array(argsort(multiply(A,A).sum(axis=0)[0]))[0]
    W = W[keys,:]
    W = W[::-1,:]     # % Is this smart ?
    
    
    if verbose:
        print("TD4 -> Fixing the signs", file=stdout)
    b	= W[:,0]
    signs = array(sign(sign(b)+0.1).T)[0] # just a trick to deal with sign=0
    W = diag(signs) * W
    print(W.shape)
    return W

    """
    
    Main references:
    @article{Special Section on BSS: Independent Component Analysis and Signal Separation
      title 	= "Robust Independent Component Analysis via Time-Delayed Cumulant Functions",
      author       = "Pando Georgiev and Andrzej Cichocki",
      HTML 	= "http://search.ieice.org/bin/summary.php?id=e86-a_3_573",
      journal      = "IEE Proceedings-F",
      month = march, number = 3, pages = {573-580}, volume = E-86-A, year = 2003}
      
      Notes:
      ** Here we consider signals which are spatially and temporally decorrelated of 
      order 2
      
      1) Blind Source Separation problem of signals which are temporally correlated of 
      order 4 can be converted to a symmetric eigenvalue problem of generalized cumulant 
      matrix CM. The measure of independence corresponds to the 'diagonality' of a set of cumulant matrices. 
      Joint diagonalization of these cumulant matrices is done depending on time delay.
      
      2) We expect to use small number of significant eigenmatrices to efficiently 
      summarize all the 4th order information. 
      
      3) Since we are dealing with real signals, it becomes easy to exploit the 
      symmetries of the cumulants to further reduce the number of matrices to be 
      diagonalized.
      
      4) Computational errors can destroy the symmetricity of the cumulant matrices Qij and thus it's restored by performing
      Qij = 0.5*[Qij + Qij.T]
      
      5) The rows of the separating matrix Btd4 are resorted in such a way that the
      columns of the corresponding mixing matrix A=pinv(Btd4) are in decreasing order of (Euclidian)
      norm.  This is a simple, `almost canonical" way of fixing the indetermination of
      permutation.  It has the effect that the first rows of the recovered signals (ie the
      first rows of S = Btd4*data) correspond to the most energetic *components*.  Recall however that
      the source signals in S=Btd4 * data have unit variance.  Therefore, when we say that the
      observations are unmixed in order of decreasing energy, this energetic signature is to
      be found as the norm of the columns of A=pinv(Btd4) and not as the variances of the
      separated source signals.
      
      """



# -*- coding: utf-8 -*-

"""
TD2

This module contains only one function, TD2, which performs temporal decorrelation of 2nd order.
"""


from __future__ import print_function

import numpy as np
from sys import stdout
from numpy import argsort, cov, diag, dot, sqrt, std, float64, mean,shape, ndarray, matrix, absolute, arange, log, tanh, pi,abs
from numpy.linalg import inv, eigh, eig
from numpy import sqrt
import warnings


def TD2(Y, m=None, U=None, lag=None, verbose=True):
    
    """
    TD2 - Temporal Decorrelation of 2nd order of real signals 
    
    Parameters:
    
        Y -- an m x T spatially whitened matrix (m: dimensioanlity of subspace, T snapshots). May be a numpy 
                array or matrix where 
                
				m: dimensionality of the subspace we are interested in. Defaults to None, in
                which case m=n.
                T: number of snapshots of MD trajectory			
      
        U -- whitening matrix obtained after doing the PCA analysis on m components
              of real data
        
        lag -- lag time in the form of an integer denoting the time steps
        
        verbose -- print info on progress. Default is True.
    
    Returns:
    
        V -- An n x m matrix V (NumPy matrix type) is a separating matrix such that
        V = Btd2 x U (U is obtained from SD2 of data matrix and Btd2 is obtained from 
        time-delayed covariance of matrix Y)
        
        Z -- Z = B2td2 * Y is spatially whitened and temporally decorrelated
        (2nd order) source extracted from the m x T spatially whitened matrix Y.         
         
        Dstd2: has eigen values sorted by increasing variance
		PCstd2: holds the index for m most significant principal components by decreasing variance
        
		R = Dstd2[PCstd2]
		R    -- Eigen values of the time-delayed covariance matrix of Y 
        Btd2 -- Eigen vectors of the time-delayed covariance matrix of Y
        
        Quick notes (more at the end of this file):
        
        o TD2 is performed on a spatially decorrelated real valued signal.
          This code makes use of AMUSE algorithm which can be found using the link:
          http://docs.markovmodel.org/lecture_tica.html
          
        o Tic is the time lagged covariance matrix computed with time lag = lag
          To ensure symmetricity of the covariance matrix a mathematical computation is 
          made: Tic = 0.5*[Tic + Tic.T]
          
        o Eigen value decomposition of this time delayed symmetrized covariance matrix 
          is performed to obtain eigen vector matrix  
        """
    # GB: we do some checking of the input arguments and copy data to new
    # variables to avoid messing with the original input. We also require double
    # precision (float64) and a numpy matrix type for Y.
    warnings.simplefilter("ignore", np.ComplexWarning)
    assert isinstance(Y, ndarray),\
        "Y (input data matrix) is of the wrong type (%s)" % type(Y)
    origtype = Y.dtype # remember to return matrix B of the same type
    Y = matrix(Y.astype(float64))
    assert Y.ndim == 2, "Y has %d dimensions, should be 2" % Y.ndim
    assert (verbose == True) or (verbose == False), \
        "verbose parameter should be either True or False"
    
    [n,T] = Y.shape # GB: n is number of input signals, T is number of samples
    
    if m==None:
        m=n 	# Number of sources defaults to # of sensors
    assert m<=n,\
        "TD2 -> Do not ask more sources (%d) than sensors (%d )here!!!" % (m,n)
    Y = Y.T
    Y1 = Y[0: T-lag:]
    Y2 = Y[lag:T:]
    if verbose:
        print("2nd order Temporal Decorrelation -> Looking for %d sources" % m, file=stdout)
        print("2nd order Temporal Decorrelation -> Removing the mean value", file=stdout)
    
    #compute time-delayed covariance matrix
    Tic = (Y1.T * Y2)/float(T-lag)
    Tic = ((Tic + Tic.T))/2
    
    if verbose:
        print("2nd order Temporal Decorrelation -> Whitening the data", file=stdout)

    [Dtd2,Utd2] = eig((Tic)) # An eigen basis for the sample covariance matrix
    ktd2 = abs(Dtd2).argsort()
    Dstd2 = Dtd2[ktd2] # sorting by increasing variance
    PCstd2 = arange(n-1, n-m-1, -1)      
    Btd2 = Utd2[:,ktd2[PCstd2]].T 
    R = Dstd2[PCstd2]
    V = dot(Btd2,U)
    # --- Scaling  ------------------------------------------------------
    scales = sqrt(abs(R)) # The scales of the principal components .
    B2td2 = dot(diag(1./scales) , Btd2)  
    
    Z = dot( B2td2 , Y.T ) # B2td2 is a whitening matrix in temporal domain and Z is 
    # spatially whitened and temporally decorrelated
    return (Z,R,Btd2,V)
    
    """
    NOTE: At this stage, Z is obtained by projecting the trajectory obtained from Y
    onto the dominant eigen vectors
    Z is now a matrix of spatially whitened and temporally uncorrelated components in the 2nd order
    """
    



# -*- coding: utf-8 -*-


from __future__ import print_function

import math
import numpy
import numpy.linalg
import MDAnalysis

from mpl_toolkits.mplot3d import Axes3D
import matplotlib.pyplot as plt

class KabschAlign(object):

	def __init__(self):
		"""
		Constructor
		"""

	def kabsch(self, toXYZ, fromXYZ):
		"""
		Input is a 3 x N array of coordinates.
		"""
		# This file has been edited to produce identical results as the original matlab implementation.
		len1 = numpy.shape(fromXYZ);
		len2 = numpy.shape(toXYZ);

		if not(len1[1] == len2[1]):
			print('KABSCH: unequal array sizes');
			return;

		m1 = numpy.mean(fromXYZ, 1).reshape((len1[0],1)); # print numpy.shape(m1);
		m2 = numpy.mean(toXYZ, 1).reshape((len2[0],1));

		tmp1 = numpy.tile(m1,len1[1]);
		tmp2 = numpy.tile(m1,len2[1]);

		assert numpy.allclose(tmp1, tmp2);
		assert tmp1.shape == fromXYZ.shape;
		assert tmp2.shape == toXYZ.shape;

		t1 = fromXYZ - tmp1;
		t2 = toXYZ - tmp2;

		[u, s, wh] = numpy.linalg.svd(numpy.dot(t2,t1.T));
		w = wh.T;

		R = numpy.dot(numpy.dot(u,[[1, 0, 0],[0, 1, 0],[0, 0, numpy.linalg.det(numpy.dot(u,w.T))]]), w.T);


		T = m2 - numpy.dot(R,m1);
		tmp3 = numpy.reshape(numpy.tile(T,(len2[1])),(len1[0],len1[1]));

		err = toXYZ - numpy.dot(R,fromXYZ) - tmp3;

		#eRMSD = math.sqrt(sum(sum((numpy.dot(err,err.T))))/len2[1]);
		eRMSD = math.sqrt(sum(sum(err**2))/len2[1]);
		return (R, T, eRMSD, err.T);

	def wKabschDriver(self, toXYZ, fromXYZ, sMed=1.5, maxIter=20):
		scaleMed = sMed;
		weights = numpy.ones( numpy.shape(toXYZ)[1] ); #print 'weights: ', numpy.shape(weights);
		flagOut = 0;
		Rc = []; Tc = []; sigc = [];
		for itr in range(0, maxIter):
			[R, T, eRMSD, err] = self.wKabsch(toXYZ, fromXYZ, weights);
			Rc.append(R);
			Tc.append(T);
			tmp1 = numpy.reshape(numpy.tile(T, (numpy.shape(toXYZ[1]))), (numpy.shape(toXYZ)[0],numpy.shape(toXYZ)[1]));
			deltaR = numpy.array( numpy.dot(R, fromXYZ) + tmp1 - toXYZ ); #print 'deltaR shape: ', numpy.shape(deltaR);
			#print deltaR;
			#numpy.save('deltaR.npy', deltaR);
			nDeltaR = numpy.sqrt(numpy.sum(deltaR**2, axis = 0)); #print 'nDeltaR shape:', numpy.shape(nDeltaR);
			sig = scaleMed*numpy.median(nDeltaR);
			sigc.append(sig);
			weights = (sig**2)/((sig**2 + nDeltaR**2)**2); #print numpy.shape(weights);
		return ( R, T, eRMSD, err);

	def wKabsch(self, toXYZ, fromXYZ, weights):
		len1 = numpy.shape(fromXYZ); #print 'len1: ', len1;
		len2 = numpy.shape(toXYZ); #print 'len2: ', len2;

		if not(len1[1] == len2[1]):
			print('wKABSCH: unequal array sizes');
			return;

		#if not (numpy.shape(weights)[0]==1):
		#	print 'am here:'
		#	weights = weights.T;

		dw = numpy.tile(weights, (3,1)); #print 'dw shape:', numpy.shape(dw);
		wFromXYZ = dw * fromXYZ; #print 'wFromXYZ shape: ', numpy.shape(wFromXYZ);
		wToXYZ = dw * toXYZ; # print 'wToXYZ shape: ', numpy.shape(wToXYZ);

		m1 = numpy.sum(wFromXYZ, 1) / numpy.sum(weights); #print numpy.shape(m1);
		m2 = numpy.sum(wToXYZ, 1) / numpy.sum(weights); #print numpy.shape(m2);

		tmp1 = numpy.reshape(numpy.tile(m1,(len1[1])), (len1[0],len1[1]));
		tmp2 = numpy.reshape(numpy.tile(m2,(len2[1])), (len2[0],len2[1]));
		t1 = numpy.reshape(fromXYZ - tmp1, (len1[0], len1[1])); #print 't1 shape: ', numpy.shape(t1);
		t2 = numpy.reshape(toXYZ - tmp2, (len2[0],len2[1]));

		aa = numpy.zeros((3,3));
		for i in range(0, numpy.shape(t1)[1]):
			tmp = numpy.outer(t2[:,i],t1[:,i]); #print 'tmp shape: ', numpy.shape(tmp);
			aa = aa + numpy.multiply(weights[i], tmp);
		aa = aa/numpy.sum(weights);

		[u,s,wh] = numpy.linalg.svd(aa);
		w = wh.T;

		R = numpy.dot(numpy.dot(u,[[1, 0, 0],[0, 1, 0],[0, 0, numpy.linalg.det(numpy.dot(u,w.T))]]), w.T);


		T = m2 - numpy.dot(R,m1);
		#print numpy.shape(m2), numpy.shape(R), numpy.shape(m1);

		tmp3 = numpy.reshape(numpy.tile(T,(len2[1])),(len1[0],len1[1]));

		err = toXYZ - numpy.dot(R,fromXYZ) - tmp3;

		#eRMSD = math.sqrt(sum(sum((numpy.dot(err,err.T))))/len2[1]);
		eRMSD = math.sqrt(sum(sum(err**2))/len2[1]);
		return (R, T, eRMSD, err.T);



if __name__ == '__main__':
	u = MDAnalysis.Universe('../data/ubq_1111.pdb', '../data/UBQ_500ns.dcd', permissive=False);
	ca = u.selectAtoms('name CA');
	f1 = ca.coordinates();
	ka = KabschAlign(); RMS = []; frames = [];
	for ts in u.trajectory:
		f2 = ca.coordinates();
		[R, T, eRMSD, err] = ka.kabsch(f2.T, f1.T);
		RMS.append(eRMSD);
		frames.append(ts.frame);

	rmsRef = [];
	f = open('../data/rms.log', 'r');
	for line in f.readlines():
		l = line.strip().split();
		rmsRef.append(float(l[1]));
	f.close();

	print((numpy.corrcoef(rmsRef, RMS)));

	fig = plt.figure();
	ax = fig.add_subplot(111);
	ax.plot(frames, RMS, 'r-', frames, rmsRef, 'b--', linestyle='solid', linewidth=2.0);

	plt.show();
